# zyx ==============================================================================

__all__ = [
    "audio",
    "image",
    "BaseModel",
    "Field",
    "logger",
    "rich_console",
    "batch",
    "lightning",
    "completion",
    "embeddings",
    "cast",
    "extract",
    "classify",
    "function",
    "generate",
    "inference",
    "paint",
    "speak",
    "transcribe",
    "tailwind",
]

import builtins as _builtins
from rich import print as _print

_builtins.print = _print

from .client import _completion as completion
from .client import _embeddings as embeddings
from .client import _cast as cast
from .client import _extract as extract
from .client import _classify as classify
from .client import _function as function
from .client import _generate as generate
from .client import _inference as inference
from .core import BaseModel as BaseModel
from .core import Field as Field
from .core import _logger as logger
from .core import _rich_console as rich_console
from .core import _batch as batch
from .core import _lightning as lightning
from .image import paint as paint
from .audio import speak as speak
from .audio import transcribe as transcribe
from .notebook import tailwind as tailwind

from . import audio as audio
from . import image as image
