# zyx ==============================================================================

__all__ = [
    "Audio",
    "speak",
    "transcribe",
]

from marvin.ai.audio import speak as speak
from marvin.ai.audio import transcribe as transcribe
from marvin.ai.audio import Audio as Audio
